package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.WatchListEntry;
import at.ac.uibk.gitsearch.repository.jpa.WatchListEntryRepository;
import at.ac.uibk.gitsearch.service.dto.WatchListEntryDTO;
import at.ac.uibk.gitsearch.service.mapper.WatchListEntryMapper;
import java.util.List;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service Implementation for managing {@link WatchListEntry}.
 */
@Service
@Transactional
public class WatchListEntryService {

    private final Logger log = LoggerFactory.getLogger(WatchListEntryService.class);

    private final WatchListEntryRepository watchListEntryRepository;

    private final WatchListEntryMapper watchListEntryMapper;

    public WatchListEntryService(WatchListEntryRepository watchListEntryRepository, WatchListEntryMapper watchListEntryMapper) {
        this.watchListEntryRepository = watchListEntryRepository;
        this.watchListEntryMapper = watchListEntryMapper;
    }

    /**
     * Save a watchListEntry.
     *
     * @param watchListEntryDTO the entity to save.
     * @return the persisted entity.
     */
    public WatchListEntryDTO save(WatchListEntryDTO watchListEntryDTO) {
        log.debug("Request to save WatchListEntry : {}", watchListEntryDTO);
        WatchListEntry watchListEntry = watchListEntryMapper.toEntity(watchListEntryDTO);
        watchListEntry = watchListEntryRepository.save(watchListEntry);
        return watchListEntryMapper.toDto(watchListEntry);
    }

    /**
     * Get all the watchListEntries.
     *
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<WatchListEntryDTO> findAll(Pageable pageable) {
        log.debug("Request to get all WatchListEntries");
        return watchListEntryRepository.findAll(pageable).map(watchListEntryMapper::toDto);
    }

    /**
     * Get one watchListEntry by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<WatchListEntryDTO> findOne(Long id) {
        log.debug("Request to get WatchListEntry : {}", id);
        return watchListEntryRepository.findById(id).map(watchListEntryMapper::toDto);
    }

    /**
     * Delete the watchListEntry by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete WatchListEntry : {}", id);
        watchListEntryRepository.deleteById(id);
    }

    /**
     * Delete the watchListEntry by id.
     *
     * @param id the id of the entity.
     */
    public void deleteInWatchlist(Long watchListId, String exerciseId) {
        final List<WatchListEntry> entries = watchListEntryRepository.getEntriesByWatchlistAndExerciseId(watchListId, exerciseId);
        entries.forEach(en -> delete(en.getId()));
    }

    //    /**
    //     * Search for the watchListEntry corresponding to the query.
    //     *
    //     * @param query the query of the search.
    //     * @param pageable the pagination information.
    //     * @return the list of entities.
    //     */
    //    @Transactional(readOnly = true)
    //    public Page<WatchListEntryDTO> search(String query, Pageable pageable) {
    //        log.debug("Request to search for a page of WatchListEntries for query {}", query);
    //        return watchListEntrySearchRepository.search(queryStringQuery(query), pageable)
    //            .map(watchListEntryMapper::toDto);
    //    }
    //
    /**
     * Search for the watchListEntry for a watchlist.
     *
     * @param query the query of the search.
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public List<WatchListEntryDTO> getEntriesForWatchlist(Long watchlistId) {
        return watchListEntryMapper.toDto(watchListEntryRepository.getEntriesByWatchlist(watchlistId));
    }
}
