import { Injectable } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';

import { ISavedSearches, NewSavedSearches } from '../saved-searches.model';

/**
 * A partial Type with required key is used as form input.
 */
type PartialWithRequiredKeyOf<T extends { id: unknown }> = Partial<Omit<T, 'id'>> & { id: T['id'] };

/**
 * Type for createFormGroup and resetForm argument.
 * It accepts ISavedSearches for edit and NewSavedSearchesFormGroupInput for create.
 */
type SavedSearchesFormGroupInput = ISavedSearches | PartialWithRequiredKeyOf<NewSavedSearches>;

type SavedSearchesFormDefaults = Pick<NewSavedSearches, 'id'>;

/*
type SavedSearchesFormGroupContent = {
  id: FormControl;
  name: FormControl;
  jsonQuery: FormControl;
  user: FormControl;
};
*/

export type SavedSearchesFormGroup = FormGroup;

@Injectable({ providedIn: 'root' })
export class SavedSearchesFormService {
  createSavedSearchesFormGroup(savedSearches: SavedSearchesFormGroupInput = { id: null }): SavedSearchesFormGroup {
    const savedSearchesRawValue = {
      ...this.getFormDefaults(),
      ...savedSearches,
    };
    return new FormGroup({
      id: new FormControl(
        { value: savedSearchesRawValue.id, disabled: true },
        {
          // nonNullable: true,
          validators: [Validators.required],
        }
      ),
      name: new FormControl(savedSearchesRawValue.name, {
        validators: [Validators.maxLength(50)],
      }),
      jsonQuery: new FormControl(savedSearchesRawValue.jsonQuery, {
        validators: [Validators.maxLength(2000)],
      }),
      user: new FormControl(savedSearchesRawValue.user),
    });
  }

  getSavedSearches(form: SavedSearchesFormGroup): ISavedSearches | NewSavedSearches {
    return form.getRawValue() as ISavedSearches | NewSavedSearches;
  }

  resetForm(form: SavedSearchesFormGroup, savedSearches: SavedSearchesFormGroupInput): void {
    const savedSearchesRawValue = { ...this.getFormDefaults(), ...savedSearches };
    form.reset(
      {
        ...savedSearchesRawValue,
        id: { value: savedSearchesRawValue.id, disabled: true },
      } as any /* cast to workaround https://github.com/angular/angular/issues/46458 */
    );
  }

  private getFormDefaults(): SavedSearchesFormDefaults {
    return {
      id: null,
    };
  }
}
